@@ -4,7 +4,7 @@
 
 seqprecorr <- function(seqdata, state.order=alphabet(seqdata), state.equiv = NULL,
       penalized="BOTH", method="TRATEDSS", weight.type="ADD", stprec=NULL,
-      with.missing=FALSE, tr.type) {
+      with.missing=FALSE, border.effect=10, tr.type) {
 
   TraMineR.check.depr.args(alist(method = tr.type))
 
@@ -12,7 +12,8 @@ seqprecorr <- function(seqdata, state.order=alphabet(seqdata), state.equiv = NUL
 		stop(" [!] seqdata is NOT a sequence object, see seqdef function to create one")
 
   if(is.null(stprec) && method=="RANK"){
-    stprec <- suppressMessages(seqprecstart(seqdata, state.order=state.order, state.equiv=state.equiv))
+    stprec <- suppressMessages(seqprecstart(seqdata, state.order=state.order,
+                        state.equiv=state.equiv, with.missing=with.missing))
   }
 
   if (is.logical(penalized)){
@@ -23,13 +24,14 @@ seqprecorr <- function(seqdata, state.order=alphabet(seqdata), state.equiv = NUL
   else
     seqprecorr.tr(seqdata, state.order=state.order, state.equiv = state.equiv,
       method=method, weight.type=weight.type, penalized=penalized,
-      stprec=stprec, with.missing=with.missing)
+      stprec=stprec, with.missing=with.missing, border.effect = border.effect)
 
 }
 
 
 seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv = NULL,
-      method="TRATEDSS", weight.type="ADD", penalized="BOTH", stprec=NULL, with.missing=FALSE) {
+      method="TRATEDSS", weight.type="ADD", penalized="BOTH", stprec=NULL, with.missing=FALSE,
+      border.effect = 10) {
 
   ## weight.type == "ADD"  : additive, i.e. 1-tr
   ##             == "INV"  : inverse, i.e. 1/tr
@@ -58,6 +60,8 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
     weight.names <- c('ADD','INV','LOGINV')
     if (!(weight.type %in% weight.names))
   		stop(" [!] weight.type should be one of ", paste(weight.names, collapse=", "))
+    if (!(border.effect > 1))
+  		stop(" [!] border.effect should be strictly greater than one!")
   }
 
   pen.names <- c('NEG','POS','BOTH','NO')
@@ -65,12 +69,14 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
 		stop(" [!] penalized should be one of ", paste(method.names, collapse=", "))
 
   ## Checking that listed states are in alphabet
-  state.order <- states.check(seqdata, state.order, state.equiv)
+  state.order <- states.check(seqdata, state.order, state.equiv, with.missing=with.missing)
 	
 	###################################
 	## setting signs according to the rank order of the states
 
-	alph=length(alphabet(seqdata))
+	alphabet <- alphabet(seqdata)
+  if (with.missing) alphabet <- c(alphabet, attr(seqdata,"nr"))
+  alph=length(alphabet)
   tr <- matrix(1, nrow=alph, ncol=alph)
   diag(tr) <- 0
   signs <- matrix(0, nrow=alph, ncol=alph)
@@ -78,8 +84,8 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
   state.noncomp <- NULL
 	
   if (length(unique(state.order)) < alph){
-    inoncomp <- which(is.na(match(alphabet(seqdata),unique(state.order))))
-    state.noncomp <- alphabet(seqdata)[inoncomp]
+    inoncomp <- which(is.na(match(alphabet,unique(state.order))))
+    state.noncomp <- alphabet[inoncomp]
     message(" [>] Non ranked states: ", paste(state.noncomp, collapse=', '))
     state.order.plus <- c(state.order, state.noncomp)
   } else {
@@ -93,6 +99,7 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
   ## since transitions from incomparable state will be ignored
 
   seqdata.ori <- seqdata ## just in case we would need the original later
+  alphabet.ori <- alphabet
 
   ##print(seqdata)	
   ##ii <- as.matrix(which(seqdata=="I", arr.ind =TRUE))
@@ -133,7 +140,8 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
   dss <- seqdss(seqdata, with.missing=with.missing)
   dssl <- seqlength(dss)
   nbseq <- nrow(dss)
-
+  ##alphabet <- alphabet(seqdata)
+  ##if (with.missing) alphabet <- c(alphabet, attr(seqdata, "nr"))
 
 	##  default tr set above as 1s
   if (method %in% c('FREQ','TRATE','TRATEDSS')) {
@@ -141,41 +149,46 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
     ## Computing transition probabilities
 
 	  if (method == "FREQ") { ## default tr set above as 1s (no transition weight)
-	    tr <- suppressMessages(seqtrate(dss, count=TRUE))  ## Here we compute the counts of the transitions
+	    tr <- suppressMessages(seqtrate(dss, count=TRUE, with.missing=with.missing))  ## Here we compute the counts of the transitions
       tr <- tr / sum(tr) ## and now the proportion of each observed transition
 	  }
 	  else if (method == "TRATEDSS") {
-	    tr <- suppressMessages(seqtrate(dss))
+	    tr <- suppressMessages(seqtrate(dss, with.missing=with.missing))
 	  }
 	  else if (method == "TRATE") {
-	    tr <- suppressMessages(seqtrate(seqdata))
+	    tr <- suppressMessages(seqtrate(seqdata, with.missing=with.missing))
 	  }
 
 	  ## Computing transition weights from transition probabilities
 
-    eps <- .000001
+    eps <- 1e-10
+    ##border.effect <- 10
+    diag(tr) <- 0
+    ## adjustement when any tr close from 1
+    if (any(tr > 1 - .1/border.effect)) tr <- tr - tr/border.effect
 
 		if (weight.type == "ADD") {
 		  tr <- 1 - tr
 		}
 		else if (weight.type == "INV"){
-		  tr <- (1 + eps)/(tr + eps)
+		  tr <- (1 + eps)/(tr + eps) ## - 1  ## GR 29/04/19 minus 1 to set min at 0
 		}
 		else if (weight.type == "LOGINV"){
 		  tr <- log((1 + eps)/(tr + eps))
 		}
-  	tr <- tr/tr[1,1] ## normalize by diagonal value
+  	##tr <- tr/tr[1,1] ## normalize by diagonal value
+  	tr <- tr/diag(tr) ## normalize by diagonal value
   }
   else if (method == "RANK"){
       for (j in 1:length(stprec))
         tr[,j] <- abs(stprec - stprec[j])
-      rownames(tr) <- colnames(tr) <- alphabet(seqdata)
+      rownames(tr) <- colnames(tr) <- alphabet ##(seqdata)
   }
 
 	## reorder rows and colunms according to state.order
 	
-	ord <- match(state.order.plus,alphabet(seqdata.ori))
-	ordo <- match(alphabet(seqdata.ori),state.order.plus)
+	ord <- match(state.order.plus,alphabet.ori)
+	ordo <- match(alphabet.ori,state.order.plus)
 
   #tr <- tr[ord,ord]
 	signs <- signs[ord,ord]	
@@ -198,7 +211,7 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
 	if(!is.null(state.equiv)){
 	  lc <- length(state.equiv)
 	  for (i in 1:lc) {
-	    iequiv <- match(state.equiv[[i]],alphabet(seqdata))
+	    iequiv <- match(state.equiv[[i]],alphabet)
 	    signs[iequiv,iequiv] <- 0
 	    tr[iequiv,iequiv] <- 0
 	  }
@@ -219,9 +232,10 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
 	rownames(transw) <- rownames(seqdata)
 	transpen <- transw
 	prop.transpen <- transw
-	
+
+
   if (penalized != 'NO') {
-	  dss.num <- TraMineR:::seqasnum(dss)+1
+	  dss.num <- TraMineR:::seqasnum(dss, with.missing=with.missing)+1
 	  ## sum of transition weights in the sequence
   	for (i in 1:nbseq) {
   		if (dssl[i]>1) {
@@ -242,11 +256,17 @@ seqprecorr.tr <- function(seqdata, state.order=alphabet(seqdata), state.equiv =
 	colnames(prop.transpen) <- "Penalty"
 	attr(prop.transpen,"tr") <- tr
 	attr(prop.transpen,"signs") <- signs
-	##attr(prop.transpen,"ord") <- ord
-	##attr(prop.transpen,"ordo") <- ordo
 	attr(prop.transpen,"state.noncomp") <- state.noncomp
 	attr(prop.transpen,"state.order") <- state.order.plus
 	##attr(prop.transpen,"seqdata") <- seqdata
 	
 	return(prop.transpen)
 }
+
+print.seqprecorr <- function(x, ...){
+  names <- dimnames(x)
+  attributes(x) <- NULL
+  x <- as.matrix(x)
+  dimnames(x) <- names
+  print(x, ...)
+}
